/* 
   (C) 2004-2007 by Oskar Senft. All rights reserved. 
   
   Published under GPL.
   
   For other licenses, please contact osk@senft-net.com

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
*/

/*
 * $Id: modeminit.c 99 2003-07-16 17:34:39Z osk $
 *
 * $Log$
 * Revision 1.5  2003/07/16 17:34:39  osk
 * Changed sound-output to javax.sound
 *
 * Revision 1.4  2002/09/05 12:47:51  osk
 * Some more documentation ...
 *
 * Revision 1.3  2002/08/13 09:07:57  osk
 * Added 'modeminit /dev/ttyI?' to TTYIgetty.
 * Changed Actions-TextField behavior in Launcher.
 *
 * Revision 1.2  2002/08/13 07:35:23  osk
 * Added ID and LOG entries in modeminit.c
 *
 *
 *
 * Initialize ttyIs on Linux-System level
 * most parts are taken from isdn4kutils/vbox/modem.{c,h} (www.isdn4linux.de)
 *
 * call as "modeminit [devicename]", e. g. "modeminit /dev/ttyI2"
*/

#include <termio.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>





/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/** start of methods taken from isdn4kutils/vbox/modem.{c,h}             */

#define log            printf
#define TRUE           1
#define OK             0
#define ERROR          -1
#define returnok()     return(OK)
#define returnerror()  return(ERROR)

typedef struct termios TIO;

struct modem_info {
    int   fd;
    char* device;
};


struct modem_info modem;


/*************************************************************************/
/** modem_set_termio():	Sets the modem terminal IO.			**/
/*************************************************************************/
int modem_set_termio(TIO *modemtio)
{
	if (tcsetattr(modem.fd, TCSANOW, modemtio) >= 0) returnok();

	returnerror();
}

/*************************************************************************/
/** modem_get_termio():	Gets the modem terminal IO.			**/
/*************************************************************************/
int modem_get_termio(TIO *modemtio)
{
	if (tcgetattr(modem.fd, modemtio) >= 0) returnok();
	returnerror();
}


/*************************************************************************/
/** modem_set_termio():	Sets the modem terminal IO.			**/
/*************************************************************************/
static void	modem_set_sane_mode(TIO *modemtio, int local)
{
	modemtio->c_iflag  =  (BRKINT|IGNPAR|IXON|IXANY);
	modemtio->c_oflag  =  (OPOST|TAB3);
	modemtio->c_cflag &= ~(CSIZE|CSTOPB|PARENB|PARODD|CLOCAL);
	modemtio->c_cflag |=  (CS8|CREAD|HUPCL|(local ? CLOCAL : 0));
	modemtio->c_lflag  =  (ECHOK|ECHOE|ECHO|ISIG|ICANON);
}

/*************************************************************************/
/** modem_set_speed(): Sets the modemport speed to 57600.		**/
/*************************************************************************/
static void modem_set_speed(TIO *modemtio)
{
	log("Setting modem speed to 57600...\n");

	cfsetospeed(modemtio, B57600);
	cfsetispeed(modemtio, B57600);
}

/*************************************************************************/
/** modem_set_raw_mode(): Sets modem terminal IO to raw mode.		**/
/*************************************************************************/
static void modem_set_raw_mode(TIO *modemtio)
{
	modemtio->c_iflag		 &= (IXON|IXOFF|IXANY);
	modemtio->c_oflag		  = 0;
	modemtio->c_lflag		  = 0;
	modemtio->c_cc[VMIN]   = 1;
	modemtio->c_cc[VTIME]  = 0;
}

/*************************************************************************/
/** modem_set_flowcontrol(): Sets modem flowcontrol to hardware hand-	**/
/**	 		     shake.				        **/
/*************************************************************************/
static void modem_set_flowcontrol(TIO *modemtio)
{
	log("Setting modem flow control (HARD)...\n");

	modemtio->c_cflag &= ~(CRTSCTS);
	modemtio->c_iflag &= ~(IXON|IXOFF|IXANY);
	modemtio->c_cflag |=  (CRTSCTS);
}

/*************************************************************************/
/** modem_initialize():	Initialize the modem.				**/
/*************************************************************************/
int modem_initialize(void)
{
	TIO porttio;

	log("Initializing modem port ...\n");

	if (modem_get_termio(&porttio) != OK)
	{
		log("Can't get modem terminal IO settings (not initialized).\n");
		returnerror();
	}

	modem_set_sane_mode(&porttio, TRUE);
	modem_set_speed(&porttio);
	modem_set_raw_mode(&porttio);
	modem_set_flowcontrol(&porttio);

	if (modem_set_termio(&porttio) != OK)
	{
		log("Can't set modem terminal IO settings (not initialized).\n");
		returnerror();
	}

	returnok();
}


/*************************************************************************/
/** modem_open_port(): Opens the modem port.			        **/
/*************************************************************************/
int modem_open_port(void)
{
	log("Opening modem port \"%s\"...\n", modem.device);

	if (modem.fd == -1)
	{
		if ((modem.fd = open(modem.device, O_RDWR|O_NDELAY)) == -1)
		{
			log("Can't open modem port \"%s\".\n", modem.device);

			returnerror();
		}

		if (fcntl(modem.fd, F_SETFL, O_RDWR) == -1)
		{
			log("Can't call fcntl() to setup modem port.\n");
			
			returnerror();
		}
	}
	else log("Found open modem port (%d).\n", modem.fd);

	returnok();
}


/*************************************************************************/
/** modem_close_port():	Close modem port.				**/
/*************************************************************************/
void modem_close_port(void)
{
	log("Closing modem port (%d)...\n", modem.fd);

	close(modem.fd);

	modem.fd = -1;
}

/** end of methods taken from isdn4kutils/vbox/modem.{c,h}               */
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/










/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/** own engineering starts here (haha!)                                 */

/*************************************************************************/
/** main(...):	Open, initialize and close initialized Modem-Device	**/
/*************************************************************************/
int main(int argc, char **argv) {
    if (argc < 2) {
	log("Must specify modem-device!\n");
	exit(ERROR);
    }
    
    modem.fd = -1;
    modem.device = argv[1];
    if (modem_open_port() != OK) exit(ERROR);
    if (modem_initialize() != OK) exit(ERROR);
    modem_close_port();

    exit(OK);
}

/** own engineering stops here                                           */
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/
